import json

from .. import _helpers
from ..exception.asgardeo_auth_error import AsgardeoAuthError


class Credentials(object):
    """Base class for all Credentials objects.
    """


class TokenResponse(Credentials):
    """Credentials object for OAuth 2.0.
    """

    def __init__(self, access_token, client_id, client_secret, refresh_token,
                 token_expiry, token_uri, user_agent, revoke_uri=None,
                 id_token=None, token_response=None, scopes=None,
                 introspection_endpoint=None, id_token_jwt=None,
                 decoded_payload=None):
        """Create an instance of OAuth2Credentials.

        This constructor is instantiated by the OIDC_FLOW.

        Args:
            access_token: string, access token.
            client_id: string, client identifier.
            client_secret: string, client secret.
            refresh_token: string, refresh token.
            token_expiry: datetime, when the access_token expires.
            token_uri: string, URI of token endpoint.
            user_agent: string, The HTTP User-Agent to provide for this
                        application.
            revoke_uri: string, URI for revoke endpoint. Defaults to None; a
                        token can't be revoked if this is None.
            id_token: object, The identity of the resource owner.
            token_response: dict, the decoded response to the token request.
                            None if a token hasn't been requested yet. Stored
                            because some providers (e.g. wordpress.com) include
                            extra fields that clients may want.
            scopes: list, authorized scopes for these credentials.
            introspection_endpoint: string, the URI for the token info endpoint.
                            Defaults to None; scopes can not be refreshed if
                            this is None.
            id_token_jwt: string, the encoded and signed identity JWT. The
                          decoded version of this is stored in id_token.

        Notes:
            store: callable, A callable that when passed a Credential
                   will store the credential back to where it came from.
                   This is needed to store the latest access_token if it
                   has expired and been refreshed.
        """
        self.access_token = access_token
        self.client_id = client_id
        self.client_secret = client_secret
        self.refresh_token = refresh_token
        self.store = None
        self.token_expiry = token_expiry
        self.token_uri = token_uri
        self.user_agent = user_agent
        self.revoke_uri = revoke_uri
        self.id_token = id_token
        self.id_token_jwt = id_token_jwt
        self.token_response = token_response
        self.scopes = set(_helpers.string_to_scopes(scopes or []))
        self.introspection_endpoint = introspection_endpoint
        self.decoded_payload = decoded_payload

        # True if the credentials have been revoked or expired and can't be
        # refreshed.
        self.invalid = False


def extract_id_token(id_token):
    """Extract the JSON payload from a JWT.

    Does the extraction w/o checking the signature.

    Args:
        id_token: string or bytestring, OAuth 2.0 id_token.

    Returns:
        object, The deserialized JSON payload.
    """
    if type(id_token) == bytes:
        segments = id_token.split(b'.')
    else:
        segments = id_token.split(u'.')

    if len(segments) != 3:
        raise AsgardeoAuthError(
            'Wrong number of segments in token: {0}'.format(id_token))

    return json.loads(
        _helpers._from_bytes(_helpers._urlsafe_b64decode(segments[1])))



"""This module holds the security related functions to facilitate the asgardeo_auth_python_sdk."""

import string
import random

UNICODE_ASCII_CHARACTER_SET = string.ascii_letters + string.digits


def generate_token(length=30, chars=UNICODE_ASCII_CHARACTER_SET):
    """
    Generate a random token.

    Args:
        length : Defaults to 30
        chars : Defaults to UNICODE_ASCII_CHARACTER_SET
    """
    rand = random.SystemRandom()
    return ''.join(rand.choice(chars) for _ in range(length))



from flask import session

from ...framework import Framework


class FlaskFramework(Framework):

    def __init__(self, name="wso2_is {}"):
        self.name = name

    def set_session_data(self, request, key, value):
        sess_key = self.name.format(key)
        session[sess_key] = value

    def get_session_data(self, request, key):
        sess_key = self.name.format(key)
        return session.get(sess_key, None)

    def is_session_data_available(self, request, key):
        sess_key = self.name.format(key)
        return sess_key in session

    def clear_session_data(self, request, key):
        sess_key = self.name.format(key)
        session.pop(sess_key, None)



import logging

from flask import request as flask_req
from flask import session, redirect

from .framework import FlaskFramework
from ... import AsgardeoAuth, AsgardeoAuthError
from ...constants.common import TOKEN_RESPONSE, REDIRECT
from ...constants.token import ACCESS_TOKEN, ID_TOKEN, AUTHORIZATION_CODE, \
    STATE

logger = logging.getLogger(__name__)


class FlaskAsgardeoAuth(AsgardeoAuth):
    """
    IdentityAuth class.
    """

    def __init__(self, auth_config):
        framework = FlaskFramework()
        super().__init__(auth_config, framework)

    def prepare_params_for_workflow(self):

        constructor_kwargs = dict(
            redirect_uri=self.auth_config["logout_callback_url"],
            op_configuration=self.op_configuration,
            pkce=self.auth_config["enable_pkce"],
            prompt=self.auth_config["prompt"], code_verifier=None)
        return constructor_kwargs

    def sign_in(self):

        result = {}
        if self.framework.is_session_data_available(flask_req, ACCESS_TOKEN) \
                and self.framework.is_session_data_available(flask_req,
                                                             ID_TOKEN):
            if self.op_configuration.is_valid_op_config(
                    self.auth_config.tenant):
                result[REDIRECT] = self.auth_config.client_host
                return result
            else:
                result[TOKEN_RESPONSE] = self.oidc_flow.get_authenticated_user(
                    self.framework.get_session_data(flask_req, ID_TOKEN))
            return result
        else:
            code = self.get_authorization_code(flask_req)
            response = self.send_sign_in_request(flask_req, code)
            return response

    def get_authorization_code(self, request):
        """Retrieve parameters for fetching access token, those parameters come
        from request and previously saved temporary data in session.
        """
        code = None
        request_state = None
        if request.method == 'GET':
            if request.args.get(AUTHORIZATION_CODE):
                code = request.args[AUTHORIZATION_CODE]
                request_state = request.args.get(STATE)
        elif request.method == 'POST':
            if request.form and request.form.get(AUTHORIZATION_CODE):
                code = request.form[AUTHORIZATION_CODE]
                request_state = request.form.get(STATE)
        if code:
            self.validate_state_param(request, request_state)
            return code

        if AUTHORIZATION_CODE in session:
            return session[AUTHORIZATION_CODE]

        return None

    def validate_state_param(self, request, request_state):

        state = self.framework.get_session_data(request, STATE)
        if state != request_state:
            raise AsgardeoAuthError(
                "CSRF Warning! State not equal in request and response.")

    def send_refresh_token_request(self, refresh_token):
        self.oidc_flow.send_refresh_token_request(refresh_token)

    def sign_out(self):
        return redirect(self.send_sign_out_request())

    def is_session_data_available(self, key):
        return self.framework.is_session_data_available(None, key)



AUTHORIZATION_ENDPOINT = "authorization_endpoint"
TOKEN_ENDPOINT = "token_endpoint"
REVOKE_TOKEN_ENDPOINT = "revocation_endpoint"
END_SESSION_ENDPOINT = "end_session_endpoint"
USER_INFO_ENDPOINT = "userinfo_endpoint"
JWKS_ENDPOINT = "jwks_uri"
OP_CONFIG_INITIATED = "op_config_initiated"
INTROSPECTION_ENDPOINT = "introspection_endpoint"
TENANT = "tenant"

SERVICE_RESOURCES = {
    "authorize": "/oauth2/authorize",
    "jwks": "/oauth2/jwks",
    "logout": "/oidc/logout",
    "revoke": "/oauth2/revoke",
    "token": "/oauth2/token",
    "introspect": "/oauth2/introspect",
    "tenant": "carbon.super",
    "well_known": "/oauth2/token/.well-known/openid-configuration"
}



ACCESS_TOKEN = "access_token"
ACCESS_TOKEN_EXPIRE_IN = "expires_in"
ACCESS_TOKEN_ISSUED_AT = "issued_at"
AUTHORIZATION_CODE = "code"
STATE = "state"
ID_TOKEN = "id_token"
ID_TOKEN_JWT = "id_token_jwt"
OIDC_SCOPE = "openid"
PKCE_CODE_VERIFIER = "pkce_code_verifier"
REFRESH_TOKEN = "refresh_token"
SCOPE = "scope"
TOKEN_TYPE = "token_type"
REQUEST_PARAMS = "request_params"
ISSUER = "issuer"



"""Helper functions for commonly used utilities."""
import base64
import json
import logging
import os
import warnings

import six
from six.moves import urllib

logger = logging.getLogger(__name__)

POSITIONAL_WARNING = 'WARNING'
POSITIONAL_EXCEPTION = 'EXCEPTION'
POSITIONAL_IGNORE = 'IGNORE'
POSITIONAL_SET = frozenset([POSITIONAL_WARNING, POSITIONAL_EXCEPTION,
                            POSITIONAL_IGNORE])

positional_parameters_enforcement = POSITIONAL_WARNING

_SYM_LINK_MESSAGE = 'File: {0}: Is a symbolic link.'
_IS_DIR_MESSAGE = '{0}: Is a directory'
_MISSING_FILE_MESSAGE = 'Cannot access {0}: No such file or directory'


def scopes_to_string(scopes):
    """Converts scope value to a string.

    If scopes is a string then it is simply passed through. If scopes is an
    iterable then a string is returned that is all the individual scopes
    concatenated with spaces.

    Args:
        scopes: string or iterable of strings, the scopes.

    Returns:
        The scopes formatted as a single string.
    """
    if isinstance(scopes, six.string_types):
        return scopes
    else:
        return ' '.join(scopes)


def string_to_scopes(scopes):
    """Converts stringifed scope value to a list.

    If scopes is a list then it is simply passed through. If scopes is an
    string then a list of each individual scope is returned.

    Args:
        scopes: a string or iterable of strings, the scopes.

    Returns:
        The scopes in a list.
    """
    if not scopes:
        return []
    elif isinstance(scopes, six.string_types):
        return scopes.split(' ')
    else:
        return scopes


def parse_unique_urlencoded(content):
    """Parses unique key-value parameters from urlencoded content.

    Args:
        content: string, URL-encoded key-value pairs.

    Returns:
        dict, The key-value pairs from ``content``.

    Raises:
        ValueError: if one of the keys is repeated.
    """
    urlencoded_params = urllib.parse.parse_qs(content)
    params = {}
    for key, value in six.iteritems(urlencoded_params):
        if len(value) != 1:
            msg = ('URL-encoded content contains a repeated value:'
                   '%s -> %s' % (key, ', '.join(value)))
            raise ValueError(msg)
        params[key] = value[0]
    return params


def update_query_params(uri, params):
    """Updates a URI with new query parameters.

    If a given key from ``params`` is repeated in the ``uri``, then
    the URI will be considered invalid and an error will occur.

    If the URI is valid, then each value from ``params`` will
    replace the corresponding value in the query parameters (if
    it exists).

    Args:
        uri: string, A valid URI, with potential existing query parameters.
        params: dict, A dictionary of query parameters.

    Returns:
        The same URI but with the new query parameters added.
    """
    parts = urllib.parse.urlparse(uri)
    query_params = parse_unique_urlencoded(parts.query)
    query_params.update(params)
    new_query = urllib.parse.urlencode(query_params)
    new_parts = parts._replace(query=new_query)
    return urllib.parse.urlunparse(new_parts)


def validate_file(filename):
    if os.path.islink(filename):
        raise IOError(_SYM_LINK_MESSAGE.format(filename))
    elif os.path.isdir(filename):
        raise IOError(_IS_DIR_MESSAGE.format(filename))
    elif not os.path.isfile(filename):
        warnings.warn(_MISSING_FILE_MESSAGE.format(filename))


def _to_bytes(value, encoding='ascii'):
    """Converts a string value to bytes, if necessary.

    Unfortunately, ``six.b`` is insufficient for this task since in
    Python2 it does not modify ``unicode`` objects.

    Args:
        value: The string/bytes value to be converted.
        encoding: The encoding to use to convert unicode to bytes. Defaults
                  to "ascii", which will not allow any characters from ordinals
                  larger than 127. Other useful values are "latin-1", which
                  which will only allows byte ordinals (up to 255) and "utf-8",
                  which will encode any unicode that needs to be.

    Returns:
        The original value converted to bytes (if unicode) or as passed in
        if it started out as bytes.

    Raises:
        ValueError if the value could not be converted to bytes.
    """
    result = (value.encode(encoding)
              if isinstance(value, six.text_type) else value)
    if isinstance(result, six.binary_type):
        return result
    else:
        raise ValueError('{0!r} could not be converted to bytes'.format(value))


def _from_bytes(value):
    """Converts bytes to a string value, if necessary.

    Args:
        value: The string/bytes value to be converted.

    Returns:
        The original value converted to unicode (if bytes) or as passed in
        if it started out as unicode.

    Raises:
        ValueError if the value could not be converted to unicode.
    """
    result = (value.decode('utf-8')
              if isinstance(value, six.binary_type) else value)
    if isinstance(result, six.text_type):
        return result
    else:
        raise ValueError(
            '{0!r} could not be converted to unicode'.format(value))


def _urlsafe_b64encode(raw_bytes):
    raw_bytes = _to_bytes(raw_bytes, encoding='utf-8')
    return base64.urlsafe_b64encode(raw_bytes).rstrip(b'=')


def _urlsafe_b64decode(b64string):
    # Guard against unicode strings, which base64 can't handle.
    b64string = _to_bytes(b64string)
    padded = b64string + b'=' * (4 - len(b64string) % 4)
    return base64.urlsafe_b64decode(padded)


def parse_exchange_token_response(content):
    """Parses response of an exchange token request.

    Most providers return JSON but some (e.g. Facebook) return a
    url-encoded string.

    Args:
        content: The body of a response

    Returns:
        Content as a dictionary object. Note that the dict could be empty,
        i.e. {}. That basically indicates a failure.
    """
    resp = {}
    content = _from_bytes(content)
    try:
        resp = json.loads(content)
    except Exception:
        # different JSON libs raise different exceptions,
        # so we just do a catch-all here
        resp = parse_unique_urlencoded(content)

    # some providers respond with 'expires', others with 'expires_in'
    if resp and 'expires' in resp:
        resp['expires_in'] = resp.pop('expires')

    return resp



AUTHORIZATION_CODE_TYPE = 'authorization_code'
BASIC_TYPE = 'basic'

LOGIN_SCOPE = 'internal_login'
HUMAN_TASK_SCOPE = 'internal_humantask_view'
DEFAULT_SUPER_TENANT = 'carbon.super'
ISSUER = 'issuer'

CODE = 'code'
TRUE_STRING = 'true'

AUTHORIZATION_ENDPOINT = 'authorization_endpoint'
TOKEN_ENDPOINT = 'token_endpoint'
REVOKE_TOKEN_ENDPOINT = 'revoke_token_endpoint'
END_SESSION_ENDPOINT = 'end_session_endpoint'
JWKS_ENDPOINT = 'jwks_uri'
OP_CONFIG_INITIATED = 'op_config_initiated'
TENANT = 'tenant'

BASE_URI = 'https://localhost:9443'
AUTH_URI = '/oauth2/authorize'
DEVICE_URI = 'https://oauth2.googleapis.com/device/code'
REVOKE_URI = 'https://oauth2.googleapis.com/revoke'
TOKEN_URI = '/oauth2/token'
TOKEN_INFO_URI = 'https://oauth2.googleapis.com/tokeninfo'

REDIRECT = 'redirect'
USER = 'user'
TOKEN_RESPONSE = 'token_response'
URL = 'url'
CERTIFICATE_PATH = "certificate_path"



import logging

from .constants.common import AUTHORIZATION_CODE_TYPE, LOGIN_SCOPE, \
    DEFAULT_SUPER_TENANT, TOKEN_RESPONSE, REDIRECT, URL, USER
from .constants.token import STATE, ACCESS_TOKEN, ID_TOKEN, ID_TOKEN_JWT
from .constants.user import USERNAME
from .exception.asgardeo_auth_error import AsgardeoAuthError
from .models.auth_config import AuthConfig
from .models.authenticated_user import AuthenticatedUser
from .models.op_Configuration import OPConfiguration
from .models.token_response import TokenResponse
from .oidc_flow import OIDCFlow

logger = logging.getLogger(__name__)

DefaultConfig = {
    "authorization_type": AUTHORIZATION_CODE_TYPE,
    "client_secret": None,
    "consent_denied": False,
    "enable_pkce": True,
    "response_mode": None,
    "scope": [LOGIN_SCOPE],
    "tenant": DEFAULT_SUPER_TENANT,
    "tenant_path": "",
    "prompt": ""
}

post_auth_session_keys = [
    USER, ACCESS_TOKEN, ID_TOKEN, ID_TOKEN_JWT,
    USERNAME
]


class IdentityAuthBase(type):
    """
    Define an Instance operation that lets clients access its unique
    instance.
    """

    def __init__(cls, name, bases, attrs, **kwargs):
        super().__init__(name, bases, attrs)
        cls._instance = None

    def __call__(cls, *args, **kwargs):
        if cls._instance is None:
            cls._instance = super().__call__(*args, **kwargs)
        return cls._instance


class AsgardeoAuth(metaclass=IdentityAuthBase):
    """
    "Registry for oauth clients.
    """

    def __init__(self, auth_config, framework):

        self.framework = framework
        self.auth_config = AuthConfig(auth_config)
        self.op_configuration = OPConfiguration(
            self.auth_config)
        self.credentials = None
        self.oidc_flow = OIDCFlow(self.auth_config,
                                  self.op_configuration)
        self.token_response: TokenResponse = None

    def prepare_params_for_workflow(self):

        constructor_kwargs = dict(
            redirect_uri=self.auth_config["logout_callback_url"],
            op_configuration=self.op_configuration,
            pkce=self.auth_config["enable_pkce"],
            prompt=self.auth_config["prompt"], code_verifier=None)
        return constructor_kwargs

    def sign_in(self, request):
        """Let the framework do the logic and call the send_sign_in_request 
        with code The logic should be as follows:
        Check whether the session available and ID token available
        if yes :
            return the user details, id token etc.
        else:
            return calling the "send_sign_in_request"

        Before calling the send_sign_in_request you must ensure you extracted 
        the code from the request. Should check whether request has the code. 
        """
        raise NotImplementedError()

    def send_sign_in_request(self, request, code):

        result = {}
        if code:
            self.token_response = self.oidc_flow.send_token_request(code)
            authenticated_user = self.get_authenticated_user(
                self.token_response.decoded_payload)
            self.set_post_auth_session_data(user=authenticated_user.get_user(),
                                            username=authenticated_user.username,
                                            access_token=self.token_response.access_token,
                                            id_token=self.token_response.id_token,
                                            id_token_jwt=self.token_response.id_token_jwt)
            result[TOKEN_RESPONSE] = self.token_response, authenticated_user
        else:
            response = self.oidc_flow.send_authorization_request()
            self.save_authorize_data(request=request,
                                     redirect_uri=self.auth_config.login_callback_url,
                                     **response)
            result[REDIRECT] = response[URL]
        return result

    def get_authorization_code(self, request):
        """Retrieve parameters for fetching access token, those parameters come
        from request and previously saved temporary data in session.
        """
        raise NotImplementedError()

    def save_authorize_data(self, request, **kwargs):
        """Save temporary data into session for the authorization step. These
        data can be retrieved later when fetching access token.
        """
        logger.debug('Saving authorize data: {!r}'.format(kwargs))
        keys = [
            'redirect_uri', 'request_token',
            'state', 'code_verifier', 'nonce'
        ]
        for k in kwargs:
            if k in keys:
                self.framework.set_session_data(request, k, kwargs[k])

    def validate_state_param(self, request, request_state):

        state = self.framework.get_session_data(request, STATE)
        if state != request_state:
            raise AsgardeoAuthError("CSRF Warning! State not equal in request "
                                    "and response.")

    def get_authenticated_user(self, decoded_payload):

        params = {
            "display_name": decoded_payload.get("preferred_username",
                                                decoded_payload.get("sub",
                                                                    None)),
            "email": decoded_payload.get("email", None),
            "username": decoded_payload.get("sub", None)
        }

        return AuthenticatedUser(**params)

    def send_refresh_token_request(self, refresh_token):
        self.oidc_flow.send_refresh_token_request(refresh_token)

    def set_post_auth_session_data(self, **kwargs):
        """Save temporary data into session for the User information and the
        token. These data can be retrieved later when fetching access token.
        """
        for k in kwargs:
            if k in post_auth_session_keys:
                self.framework.set_session_data(None, k, kwargs[k])

    def clear_post_auth_session_data(self):
        """Save temporary data into session for the User information and the
        token. These data can be retrieved later when fetching access token.
        """
        for k in post_auth_session_keys:
            self.framework.clear_session_data(None, k)

    def get_post_auth_session_data(self):
        """Save temporary data into session for the User information and the
        token. These data can be retrieved later when fetching access token.
        """
        result = {}
        for k in post_auth_session_keys:
            result[k] = self.framework.get_session_data(None, k)
        return result

    def sign_out(self):
        """Let the framework do the logic and call the sign_out()
        """
        raise NotImplementedError()

    def send_sign_out_request(self):
        id_token = self.framework.get_session_data(None, ID_TOKEN_JWT)
        if not id_token:
            return self.auth_config.login_callback_url
        self.clear_post_auth_session_data()
        return self.oidc_flow.get_logout_url(id_token=id_token)



class AuthenticatedUser:

    def __init__(self, display_name, email, username):
        self.display_name: str = display_name
        self.email: str = email
        self.username: str = username

    def get_user(self):
        return self.__dict__



import http

import requests

from ..constants.common import ISSUER
from ..constants.endpoints import AUTHORIZATION_ENDPOINT, TOKEN_ENDPOINT, \
    JWKS_ENDPOINT, REVOKE_TOKEN_ENDPOINT, END_SESSION_ENDPOINT, \
    SERVICE_RESOURCES, INTROSPECTION_ENDPOINT
from ..exception.asgardeo_auth_error import AsgardeoAuthError


class OPConfiguration:

    def __init__(self, auth_config):

        self.op_config_initiated = False
        serverHost = auth_config.server_origin + auth_config.tenant_path
        well_known_url = serverHost + SERVICE_RESOURCES["well_known"]
        try:
            resp = requests.get(url=well_known_url,
                                verify=auth_config.certificate_path)
            if resp.status_code != http.client.OK:
                raise AsgardeoAuthError(
                    "Failed to load OpenID provider configuration from: " + well_known_url)

            resp_data = resp.json()
            self.authorization_endpoint = resp_data[AUTHORIZATION_ENDPOINT]
            self.token_endpoint = resp_data[TOKEN_ENDPOINT]
            self.end_session_endpoint = resp_data[END_SESSION_ENDPOINT]
            self.jwks_uri = resp_data[JWKS_ENDPOINT]
            self.revocation_endpoint = resp_data[REVOKE_TOKEN_ENDPOINT]
            self.introspection_endpoint = resp_data[INTROSPECTION_ENDPOINT]
            self.tenant = auth_config.tenant
            self.issuer = resp_data[ISSUER]
            self.op_config_initiated = True

        except:
            self.authorization_endpoint = serverHost + SERVICE_RESOURCES[
                "authorize"]
            self.token_endpoint = serverHost + SERVICE_RESOURCES["token"]
            self.end_session_endpoint = serverHost + SERVICE_RESOURCES["logout"]
            self.jwks_uri = serverHost + SERVICE_RESOURCES["jwks"]
            self.revocation_endpoint = serverHost + SERVICE_RESOURCES["revoke"]
            self.introspection_endpoint = serverHost + SERVICE_RESOURCES[
                "introspect"]
            self.tenant = SERVICE_RESOURCES["tenant"]
            self.issuer = serverHost + SERVICE_RESOURCES["token"]
            self.op_config_initiated = True
            raise AsgardeoAuthError(
                "Initialized OpenID Provider configuration from default "
                "configuration. Because failed to access wellknown endpoint: "
                "" + well_known_url)

    def reset_op_configuration(self):

        self.authorization_endpoint = None
        self.token_endpoint = None
        self.end_session_endpoint = None
        self.jwks_uri = None
        self.revocation_endpoint = None
        self.tenant = None
        self.issuer = None
        self.op_config_initiated = False

    def is_valid_op_config(self, tenant):

        return self.op_config_initiated and tenant == self.tenant



name = 'Asgardeo Auth Python SDK'
packages = ('lib', 'lib.*')
version = "0.0.53"
author = 'Asgardeo'
homepage = 'https://github.com/asgardeo/asgardeo-auth-python-sdk#readme'
license_name = 'Apache Software License'
description = "Asgardeo Auth Python SDK."
bug_tracker = 'https://github.com/asgardeo/asgardeo-auth-python-sdk/issues'
keywords = [
    "Asgardeo",
    "OIDC",
    "OAuth2",
    "Authentication",
    "Authorization"
]
download_url = 'https://github.com/asgardeo/asgardeo-auth-python-sdk/releases'
author = "Asgardeo",
author_email = "beta@asgardeo.io"
default_user_agent = '{}/{} (+{})'.format(name, version, homepage)
default_json_headers = [
    ('Content-Type', 'application/json'),
    ('Cache-Control', 'no-store'),
    ('Pragma', 'no-cache'),
]



class Framework(object):

    def __init__(self, name="wso2_is"):
        self.name = name

    def set_session_data(self, request, key, value):
        """set the session variable according to the integrated framework"""
        raise NotImplementedError()

    def get_session_data(self, request, key):
        """get the session variable according to the integrated framework"""
        raise NotImplementedError()

    def is_session_data_available(self, request, key):
        """check the availability of the session variable according to the
        integrated framework """
        raise NotImplementedError()

    def clear_session_data(self, request, key):
        """check the availability of the session variable according to the
        integrated framework """
        raise NotImplementedError()



"""This module is for holding the authentication related configs."""

from ..constants.common import AUTHORIZATION_CODE_TYPE, LOGIN_SCOPE, \
    HUMAN_TASK_SCOPE, DEFAULT_SUPER_TENANT
from ..exception.asgardeo_auth_error import AsgardeoAuthError

defaultConfig = {
    "login_callback_url",
    "logout_callback_url",
    "client_host",
    "authorization_type",
    "client_id",
    "client_secret",
    "consent_denied",
    "enable_pkce",
    "response_mode",
    "scope",
    "tenant",
    "tenant_path",
    "prompt",
    "server_origin",
    "code_verifier",
    "certificate_path"
}


class AuthConfig:
    """Base class for holding the authentication related configs.

    Store and retrieve a single credential. This class supports locking
    such that multiple processes and threads can operate on a single
    store.
    """

    def __init__(self, auth_config):

        self.scope_string = None
        self.login_callback_url = None
        self.logout_callback_url = None
        self.client_host = None
        self.authorization_type = AUTHORIZATION_CODE_TYPE
        self.client_id = None
        self.client_secret = None
        self.consent_denied = False
        self.enable_pkce = False
        self.response_mode = None
        self.scope = [LOGIN_SCOPE]
        self.tenant = DEFAULT_SUPER_TENANT
        self.tenant_path = "/t/" + DEFAULT_SUPER_TENANT
        self.prompt = ""
        self.server_origin = "https://localhost:9443"
        self.code_verifier = None
        self.certificate_path = None

        for key in auth_config:
            if key in defaultConfig:
                setattr(self, key, auth_config[key])
            else:
                raise AsgardeoAuthError(
                    "Improper key value passed in the autoconfig. Please check the auth config")



import datetime
import urllib
from venv import logger

import requests
import http as http_lib

from . import pkce, _helpers
from ._helpers import parse_exchange_token_response
from .common.security import generate_token
from .constants.token import OIDC_SCOPE
from .exception.asgardeo_auth_error import AsgardeoAuthError
from .models.auth_config import AuthConfig
from .models.crypto import validate_jwt
from .models.op_Configuration import OPConfiguration
from .models.token_response import TokenResponse, extract_id_token

_UTCNOW = datetime.datetime.utcnow


class Flow(object):
    """Base class for all Flow objects."""
    pass


class OIDCFlow(Flow):
    """This Class does the Web Server Flow .
    """

    def __init__(self, auth_config: AuthConfig = None,
                 op_configuration: OPConfiguration = None,
                 user_agent=None,
                 authorization_header=None, **kwargs):
        """Constructor for OIDCFlow. The kwargs argument is used to set extra
        query parameters on the For example, the access_type and prompt query
        parameters can be set via kwargs.

        :param auth_config: authentication related user configs.
        :param op_configuration: openid configuration
        :param user_agent: string, HTTP User-Agent to provide for this
               application.
        :param authorization_header: string, For use with OAuth 2.0 providers
               that require a client to authenticate using a header value
               instead of passing client_secret in the POST body.
        :param kwargs: dict, The keyword arguments are all optional
               and required parameters for the OAuth calls.
        """

        # scope is a required argument, but to preserve backwards-compatibility
        # we don't want to rearrange the positional arguments
        if auth_config.scope is None:
            raise TypeError("The value of scope must not be None")

        if auth_config.client_id is None:
            raise TypeError("The value of client_id must not be None")

        if auth_config.client_id is None:
            raise TypeError("The value of client_id must not be None")

        self.auth_config: AuthConfig = auth_config
        self.op_configuration: OPConfiguration = op_configuration
        if OIDC_SCOPE not in self.auth_config.scope:
            self.auth_config.scope.append(OIDC_SCOPE)

        self.auth_config.scope_string = _helpers.scopes_to_string(
            self.auth_config.scope)
        self.user_agent = user_agent
        self.authorization_header = authorization_header
        self.params = _oauth2_web_server_flow_params(kwargs)

    def get_authorize_url(self, login_callback_url=None, state=None):
        """Returns a URI to redirect to the provider.

        Args: login_callback_url: string, Either the string
        'urn:ietf:wg:oauth:2.0:oob' for a non-web-based application, or a URI
        that handles the callback from the authorization server. This
        parameter is deprecated, please move to passing the redirect_uri in
        via the constructor. state: string, Opaque state string which is
        passed through the OAuth2 flow and returned to the client as a query
        parameter in the callback.

        Returns:
            A URI as a string to redirect the user to begin the authorization
            flow.
        """
        if login_callback_url is not None:
            logger.warning((
                'The redirect_uri parameter for '
                'OAuth2WebServerFlow.step1_get_authorize_url is deprecated. '
                'Please move to passing the redirect_uri in via the '
                'constructor.'))
            self.auth_config.login_callback_url = login_callback_url

        if self.auth_config.login_callback_url is None:
            raise ValueError('The value of redirect_uri must not be None.')

        query_params = {
            'client_id': self.auth_config.client_id,
            'redirect_uri': self.auth_config.login_callback_url,
            'scope': self.auth_config.scope_string,
        }
        if state is None:
            state = generate_token()
        query_params['state'] = state

        if self.auth_config.enable_pkce:
            if not self.auth_config.code_verifier:
                self.auth_config.code_verifier = pkce.code_verifier()
            challenge = pkce.code_challenge(self.auth_config.code_verifier)
            query_params['code_challenge'] = challenge
            query_params['code_challenge_method'] = 'S256'

        query_params.update(self.params)

        return _helpers.update_query_params(
            self.op_configuration.authorization_endpoint, query_params), state

    def send_token_request(self, code=None):
        """Exchanges a code for OAuth2Credentials.

        Args:
            code: string, a dict-like object, or None. For a non-device
                  flow, this is either the response code as a string, or a
                  dictionary of query parameters to the redirect_uri. For a
                  device flow, this should be None.

        Returns:
            An OAuth2Credentials object that can be used to authorize requests.

        Raises:
            FlowExchangeError: if a problem occurred exchanging the code for a
                               refresh_token.
            ValueError: if code and device_flow_info are both provided or both
                        missing.
        """
        if code is None:
            raise ValueError('No code provided.')

        if not self.op_configuration.token_endpoint:
            raise ValueError('Invalid token endpoint found.')

        post_data = {
            'client_id': self.auth_config.client_id,
            'code': code,
            'scope': self.auth_config.scope_string,
        }
        if self.auth_config.client_secret is not None:
            post_data['client_secret'] = self.auth_config.client_secret
        if self.auth_config.enable_pkce:
            post_data['code_verifier'] = self.auth_config.code_verifier
        else:
            post_data['grant_type'] = 'authorization_code'
            post_data['redirect_uri'] = self.auth_config.login_callback_url
        body = urllib.parse.urlencode(post_data)

        headers = self.get_token_request_headers()

        resp = requests.post(self.op_configuration.token_endpoint, data=body,
                             headers=headers,
                             verify=self.auth_config.certificate_path)
        content = resp.content
        data_response = parse_exchange_token_response(content)
        if resp.status_code == http_lib.client.OK and 'access_token' in data_response:
            return self.get_token_response_from_resp(data_response)
        else:
            logger.info('Failed to retrieve access token: %s', content)
            if 'error' in data_response:
                # you never know what those providers got to say
                error_msg = (str(data_response['error']) +
                             str(data_response.get('error_description', '')))
            else:
                error_msg = 'Invalid response: {0}.'.format(str(resp.status))
            raise AsgardeoAuthError(error_msg)

    def send_authorization_request(self):

        url, state = self.get_authorize_url()
        result = {
            'url': url,
            'state': state
        }
        return result

    def get_token_request_headers(self):

        headers = {
            'content-type': 'application/x-www-form-urlencoded',
        }
        if self.authorization_header is not None:
            headers['Authorization'] = self.authorization_header
        if self.user_agent is not None:
            headers['user-agent'] = self.user_agent
        return headers

    def validate_id_token(self, id_token, client_id, issuer):

        jwks_endpoint = self.op_configuration.jwks_uri
        if not jwks_endpoint and not len(jwks_endpoint):
            raise AsgardeoAuthError("Invalid JWKS URI found.")
        resp = requests.get(url=jwks_endpoint,
                            verify=self.auth_config.certificate_path)
        try:
            if resp.status_code != http_lib.client.OK:
                raise AsgardeoAuthError(
                    "Failed to load public keys from JWKS URI " + jwks_endpoint)

            keys = resp.json()["keys"]
            return validate_jwt(id_token, keys, client_id, issuer)
        except AsgardeoAuthError:
            raise AsgardeoAuthError("Failed to validate the ID Token")

    def send_refresh_token_request(self, refresh_token):
        token_endpoint = self.op_configuration.token_endpoint
        if not token_endpoint:
            raise AsgardeoAuthError("Invalid token endpoint found.")

        post_data = {
            'client_id': self.auth_config.client_id,
            'refresh_token': refresh_token,
            'grant_type': "refresh_token",
        }

        headers = self.get_token_request_headers()

        resp = requests.post(token_endpoint, data=post_data, headers=headers,
                             verify=self.auth_config.certificate_path)
        content = resp.content
        data_response = parse_exchange_token_response(content)
        if resp.status_code != http_lib.client.OK and 'access_token' in data_response:
            raise AsgardeoAuthError(
                "Invalid status code received in the refresh token response: " + str(
                    resp.status_code))
        return self.get_token_response_from_resp(data_response)

    def get_token_response_from_resp(self, resp):
        access_token = resp['access_token']
        refresh_token = resp.get('refresh_token', None)
        if not refresh_token:
            logger.info(
                'Received token response with no refresh_token. Consider '
                "reauthenticating with prompt='consent'.")
        token_expiry = None
        if 'expires_in' in resp:
            delta = datetime.timedelta(seconds=int(resp['expires_in']))
            token_expiry = delta + _UTCNOW()

        extracted_id_token = None
        id_token_jwt = None
        if 'id_token' in resp:
            extracted_id_token = extract_id_token(resp['id_token'])
            id_token_jwt = resp['id_token']

        logger.info('Successfully retrieved access token')
        decoded_payload = self.validate_id_token(id_token_jwt,
                                                 self.auth_config.client_id,
                                                 self.op_configuration.issuer)
        return TokenResponse(
            access_token, self.auth_config.client_id,
            self.auth_config.client_secret,
            refresh_token, token_expiry, self.op_configuration.token_endpoint,
            self.user_agent,
            revoke_uri=self.op_configuration.revocation_endpoint,
            id_token=extracted_id_token,
            id_token_jwt=id_token_jwt, token_response=resp,
            scopes=self.auth_config.scope_string,
            introspection_endpoint=self.op_configuration.introspection_endpoint,
            decoded_payload=decoded_payload)

    def send_revoke_token_request(self, access_token):

        revocation_endpoint = self.op_configuration.revocation_endpoint

        if not revocation_endpoint or not len(revocation_endpoint.strip()):
            raise AsgardeoAuthError("Invalid revoke token endpoint found.")

        post_data = {
            'client_id': self.auth_config.client_id,
            'token': access_token,
            'token_type_hint': "access_token",
        }

        headers = self.get_token_request_headers()

        resp = requests.post(revocation_endpoint, data=post_data,
                             headers=headers,
                             verify=self.auth_config.certificate_path)
        if resp.status_code != http_lib.client.OK:
            raise AsgardeoAuthError(
                "Invalid status code received in the revoke token response: " + str(
                    resp.status_code))
        return resp.json()

    def get_logout_url(self, logout_callback_url=None, id_token=None):
        """Returns a URI to redirect to the provider.

        Args: login_callback_url: string, Either the string
        'urn:ietf:wg:oauth:2.0:oob' for a non-web-based application, or a URI
        that handles the callback from the authorization server. This
        parameter is deprecated, please move to passing the redirect_uri in
        via the constructor. state: string, Opaque state string which is
        passed through the OAuth2 flow and returned to the client as a query
        parameter in the callback.

        Returns:
            A URI as a string to redirect the user to begin the authorization
            flow.
        """

        logout_endpoint = self.op_configuration.end_session_endpoint

        if not logout_endpoint:
            raise AsgardeoAuthError("No logout endpoint found in the session.")

        if not id_token:
            raise AsgardeoAuthError("Invalid id_token found in the session.")

        if logout_callback_url is not None:
            logger.warning((
                'The redirect_uri parameter for '
                'OAuth2WebServerFlow.step1_get_authorize_url is deprecated. '
                'Please move to passing the redirect_uri in via the '
                'constructor.'))
            self.auth_config.logout_callback_url = logout_callback_url

        if not self.auth_config.logout_callback_url:
            raise ValueError('The value of redirect_uri must not be None.')

        query_params = {
            'post_logout_redirect_uri': self.auth_config.logout_callback_url,
            'id_token_hint': id_token
        }

        return _helpers.update_query_params(logout_endpoint, query_params)


def _oauth2_web_server_flow_params(kwargs):
    """Configures redirect URI parameters for OAuth2WebServerFlow."""
    params = {
        'access_type': 'offline',
        'response_type': 'code',
    }

    params.update(kwargs)

    # Check for the presence of the deprecated approval_prompt param and
    # warn appropriately.
    approval_prompt = params.get('approval_prompt')
    if approval_prompt is not None:
        logger.warning(
            'The approval_prompt parameter for OAuth2WebServerFlow is '
            'deprecated. Please use the prompt parameter instead.')

        if approval_prompt == 'force':
            logger.warning(
                'approval_prompt="force" has been adjusted to '
                'prompt="consent"')
            params['prompt'] = 'consent'
            del params['approval_prompt']

    return params



from distutils.core import setup
from setuptools import find_packages
from lib.app_consts import name, version, homepage, license_name, bug_tracker, \
    keywords, description, download_url, author, author_email,packages


setup(
    name=name,
    packages=find_packages(include=packages),
    version=version,
    license=license_name,
    description=description,
    author=author,
    author_email=author_email,
    url=homepage,
    download_url=download_url,
    keywords=keywords,
    install_requires=[
        'python-jose>=3.2.0',
        'Flask>=1.1.2'
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable"
        # as the current state of your package
        'Environment :: Web Environment',
        'Framework :: Flask',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: ' + license_name,
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9'
    ],
    project_urls={
        'Documentation': homepage,
        'Bug Tracker': bug_tracker,
        'Source Code': homepage,
    }
)



from jose import jwt


def get_supported_signature_algorithms():
    return ["RS256", "RS512", "RS384", "PS256"]


def validate_jwt(id_token, jwks, client_id, issuer):
    return jwt.decode(id_token,
                      jwks,
                      audience=client_id,
                      algorithms=get_supported_signature_algorithms(),
                      issuer=[issuer],
                      options={"verify_at_hash": False})



from .asgardeo_auth import AsgardeoAuth
from .constants.common import *
from .constants.user import *
from .exception.asgardeo_auth_error import AsgardeoAuthError

from .app_consts import version, homepage, author, name, license_name
__version__ = version
__homepage__ = homepage
__author__ = author
__license__ = license_name
